@@ -8,6 +8,7 @@ import textwrap
 import xmlrpclib
 
 from ConfigParser import ConfigParser
+from datetime import datetime
 
 try:
     from docutils.core import publish_parts
@@ -74,6 +75,20 @@ class ReleaseManager(object):
 
         config_parser('setup.cfg', callback)
 
+    def prereleaser_middle(self, data):
+        """Update the Sphinx conf.py"""
+
+        if data['name'] != 'pyfits':
+            return
+
+        # Get the authors out of the setup.cfg
+        cfg = ConfigParser()
+        cfg.read('setup.cfg')
+        authors = cfg.get('metadata', 'author')
+        authors = [a.strip() for a in authors.split(', ')]
+
+        update_docs_config(data['new_version'], authors)
+
     def prereleaser_after(self, data):
         """
         Before preforming the release, get the previously released version from
@@ -111,14 +126,6 @@ class ReleaseManager(object):
 
         config_parser('setup.cfg', callback)
 
-    def postreleaser_middle(self, data):
-        """
-        A postreleaser.middle hook to change the dev_version_template from the
-        annoying default of 'x.y.z.dev0' to just 'x.y.z.dev' without the 0.
-        """
-
-        data['dev_version_template'] = '%(new_version)s.dev'
-
     def postreleaser_after(self, data):
         """
         Used to update the PyFITS website.
@@ -220,6 +227,38 @@ class ReleaseManager(object):
             pass
 
 
+def update_docs_config(new_version, authors):
+    """
+    Updates the conf.py for the Sphinx documentation with the new version
+    string and an up to date authors list and copyright date.
+    """
+
+    conf_py = os.path.join('docs', 'source', 'conf.py')
+
+    with open(conf_py) as f:
+        conf_py_src = f.read()
+
+    # Update the 'copyright_year' string
+    year = datetime.now().year
+    conf_py_src = re.sub(r"^copyright_year\s*=\s*'[^']+'",
+                         'copyright_year = %r' % str(year), conf_py_src,
+                         flags=re.M)
+
+    # Update the 'authors' list
+    authors_list = ('authors = [\n    ' +
+                    ',\n    '.join(repr(a) for a in authors) + '\n]')
+    conf_py_src = re.sub(r'^authors\s*=\s*\[[^]]+\]$', authors_list,
+                         conf_py_src, flags=re.M)
+
+    # Update the version and release variables (for PyFITS we always just set
+    # these to the same)
+    conf_py_src = re.sub(r"^(version|release)\s*=\s*'[^']+'",
+                         r"\1 = %r" % new_version, conf_py_src, flags=re.M)
+
+    with open(conf_py, 'w') as f:
+        f.write(conf_py_src)
+
+
 def generate_release_notes(lines):
     """
     Generates the release notes page from the lines of restructuredText in the
@@ -263,6 +302,9 @@ def generate_release_notes(lines):
 
     content = re.sub(r'<h(\d)>([^<]+)</h\d>', increment_header, content)
 
+    # Another hackish regexp--this one to replace tt tags with code tags
+    content = re.sub(r'<tt (.*?)</tt>', r'<code \1</code>', content, re.M)
+
     # A few more quickie formatting hacks...
     content = content.splitlines()
 
