@@ -4,8 +4,8 @@ import numpy as np
 from pyfits.card import Card, CardList
 from pyfits.hdu.base import DELAYED, _ValidHDU, ExtensionHDU
 from pyfits.header import Header
-from pyfits.util import _is_pseudo_unsigned, _unsigned_zero, _is_int, \
-                        _pad_length, _normalize_slice, lazyproperty
+from pyfits.util import (_is_pseudo_unsigned, _unsigned_zero, _is_int,
+                         _pad_length, _normalize_slice, lazyproperty)
 
 class _ImageBaseHDU(_ValidHDU):
     """FITS image HDU base class.
@@ -147,6 +147,10 @@ class _ImageBaseHDU(_ValidHDU):
 
         raise NotImplementedError
 
+    @property
+    def is_image(self):
+        return True
+
     @property
     def section(self):
         """
@@ -767,7 +771,8 @@ class PrimaryHDU(_ImageBaseHDU):
         super(PrimaryHDU, self).__init__(
             data=data, header=header,
             do_not_scale_image_data=do_not_scale_image_data, uint=uint)
-        self.name = 'PRIMARY'
+
+        self._name = 'PRIMARY'
         self._extver = 1
 
         # insert the keywords EXTEND
