@@ -1,5 +1,9 @@
 from __future__ import with_statement
 
+import os
+import time
+import shutil
+
 import numpy as np
 from numpy import char as chararray
 
@@ -35,6 +39,30 @@ class TestGroupsFunctions(PyfitsTestCase):
             assert_equal(ghdu.data[0].data.shape, naxes[::-1])
             assert_equal(ghdu.data[0].parnames, parameters)
 
+    def test_open_groups_in_update_mode(self):
+        """
+        Test that opening a file containing a groups HDU in update mode and
+        then immediately closing it does not result in any unnecessary file
+        modifications.
+
+        Similar to
+        test_image.TestImageFunctions.test_open_scaled_in_update_mode().
+        """
+
+        # Copy the original file before making any possible changes to it
+        shutil.copy(self.data('random_groups.fits'),
+                    self.temp('random_groups.fits'))
+        mtime = os.stat(self.temp('random_groups.fits')).st_mtime
+
+        time.sleep(1)
+
+        pyfits.open(self.temp('random_groups.fits'), mode='update',
+                    memmap=False).close()
+
+        # Ensure that no changes were made to the file merely by immediately
+        # opening and closing it.
+        assert_equal(mtime, os.stat(self.temp('random_groups.fits')).st_mtime)
+
     def test_parnames_round_trip(self):
         """
         Regression test for #130.  Ensures that opening a random groups file in
