@@ -1,5 +1,6 @@
 import functools
 import itertools
+import mmap
 import os
 import signal
 import sys
@@ -13,8 +14,6 @@ import numpy as np
 __all__ = ['Extendable', 'register_extension', 'register_extensions',
            'unregister_extensions']
 
-from numpy import memmap as Memmap
-
 
 BLOCK_SIZE = 2880  # the FITS block size
 
@@ -790,17 +789,29 @@ def _tmp_name(input):
     return fn
 
 
-def _get_array_memmap(array):
+if sys.version_info[:2] < (2, 6):
+    # In Python 2.5 mmap.mmap is a function that returns an object of type
+    # 'mmap.mmap', but the mmap.mmap type is otherwise not accessible through
+    # the module
+    def _is_mmap(obj):
+        return (type(obj).__module__ == 'mmap' and
+                type(obj).__name__ == 'mmap')
+else:
+    def _is_mmap(obj):
+        return isinstance(obj, mmap.mmap)
+
+
+def _get_array_mmap(array):
     """
-    If the array has a numpy.memmap as one of its bases, return the memmap
-    base; otherwise return None.
+    If the array has an mmap.mmap at base of its base chain, return the mmap
+    object; otherwise return None.
     """
 
-    if isinstance(array, Memmap):
+    if _is_mmap(array):
         return array
 
     base = array
     while hasattr(base, 'base') and base.base is not None:
-        if isinstance(base.base, Memmap):
+        if _is_mmap(base.base):
             return base.base
         base = base.base
