@@ -1,4 +1,4 @@
-/* $Id: compressionmodule.c 1037 2011-08-23 21:23:12Z stsci_embray $ 
+/* $Id: compressionmodule.c 1731 2012-07-03 21:39:33Z stsci_embray $ 
 */
 
 /* "compression module */
@@ -414,6 +414,7 @@ PyObject* compression_compressData(PyObject* self, PyObject* args)
    PyObject*       tileSizeObj;
    PyObject*       zvalObj;
    PyObject*       outList;
+   PyObject*       outStr;
    PyObject*       outScale;
    PyObject*       outZero;
    PyObject*       outUncompressed;
@@ -675,9 +676,14 @@ PyObject* compression_compressData(PyObject* self, PyObject* args)
 
          for ( i = 0; i < ntiles; i++)
          {
-            PyList_Append(outList, PyString_FromStringAndSize(
-                          (const char*)((theFile.Fptr)->data[i]),
-                          (theFile.Fptr)->dataLen[i]));
+            outStr = PyString_FromStringAndSize(
+                (const char*)((theFile.Fptr)->data[i]),
+                (theFile.Fptr)->dataLen[i]);
+
+            PyList_Append(outList, outStr);
+
+            Py_DECREF(outStr);
+
             free((theFile.Fptr)->data[i]);
 
             if (cn_zscale > 0)
